==> mars-src-1.0.1.tar.gz: mars-src-1.0.1/samples/mutex/host.c <==
/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <libspe2.h>
#include <pthread.h>
#include <mars/mars.h>

#define INFO	\
"\
MARS Mutex Sample						\n\
--------------------------					\n\
This program shows the basic usage of the MARS mutex to		\n\
restrict concurrent access of some shared resource.		\n\
								\n\
This program creates %d host threads, %d tasks and creates	\n\
a single mutex to protect access of a shared resource integer	\n\
counter located in main storage. As each host thread or MPU	\n\
task runs, it locks the mutex and increments the shared		\n\
resource counter before unlocking the mutex. Since the shared	\n\
resource is protected from concurrent accesses, the resulting	\n\
value of the counter should equal to the number of total host	\n\
threads and tasks, (%d), when the program has completed.	\n\
\n", NUM_TASKS, NUM_TASKS, NUM_TASKS * 2

#define NUM_TASKS	10

extern struct spe_program_handle mpu_task_prog;

static struct mars_context *mars_ctx;
static struct mars_task_id task_id[NUM_TASKS];
static struct mars_task_args task_args;
static struct mars_mutex *mutex;
static pthread_t thread_id[NUM_TASKS];
static uint32_t shared_resource __attribute__((aligned(16)));

static void *thread(void *arg)
{
	(void)arg;
	int ret;

	ret = mars_mutex_lock(mutex);
	if (ret) {
		printf("MARS mutex lock failed! (%d)\n", ret);
		pthread_exit(&ret);
	}

	shared_resource++;

	printf("HOST  : Main() - Increment Shared Resource Counter = %d\n",
		shared_resource);

	ret = mars_mutex_unlock(mutex);
	if (ret) {
		printf("MARS mutex unlock failed! (%d)\n", ret);
		pthread_exit(&ret);
	}

	pthread_exit(&ret);

	return NULL;
}

int main(void)
{
	int ret, i;

	printf(INFO);

	ret = mars_context_create(&mars_ctx, 0, 0);
	if (ret) {
		printf("MARS context create failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_mutex_create(&mutex);
	if (ret) {
		printf("MARS mutex create failed! (%d)\n", ret);
		return 1;
	}

	shared_resource = 0;

	printf("HOST  : Main() - Shared Resource Counter = %d\n",
		shared_resource);

	for (i = 0; i < NUM_TASKS; i++) {
		char name[MARS_TASK_NAME_LEN_MAX];

		snprintf(name, MARS_TASK_NAME_LEN_MAX, "Task %d", i);
		ret = mars_task_create(mars_ctx, &task_id[i], name,
			mpu_task_prog.elf_image, NULL);
		if (ret) {
			printf("MARS task create failed! (%d)\n", ret);
			return 1;
		}

		task_args.type.u64[0] = mars_ptr_to_ea(mutex);
		task_args.type.u64[1] = mars_ptr_to_ea(&shared_resource);
		ret = mars_task_schedule(&task_id[i], &task_args, 0);
		if (ret) {
			printf("MARS task schedule failed! (%d)\n", ret);
			return 1;
		}

		pthread_create(&thread_id[i], NULL, thread, NULL);
	}

	for (i = 0; i < NUM_TASKS; i++) {
		ret = mars_task_wait(&task_id[i], NULL);
		if (ret) {
			printf("MARS task wait failed! (%d)\n", ret);
			return 1;
		}

		ret = mars_task_destroy(&task_id[i]);
		if (ret) {
			printf("MARS task destroy failed! (%d)\n", ret);
			return 1;
		}

		pthread_join(thread_id[i], NULL);
	}

	printf("HOST  : Main() - Shared Resource Counter = %d\n",
		shared_resource);

	ret = mars_mutex_destroy(mutex);
	if (ret) {
		printf("MARS mutex destroy failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_context_destroy(mars_ctx);
	if (ret) {
		printf("MARS context destroy failed! (%d)\n", ret);
		return 1;
	}

	return 0;
}

==> mars-src-1.1.0.tar.gz: mars-src-1.1.0/samples/mutex/host.c <==
/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <libspe2.h>
#include <pthread.h>
#include <mars/mars.h>

#define INFO	\
"\
MARS Mutex Sample						\n\
--------------------------					\n\
This program shows the basic usage of the MARS mutex to		\n\
restrict concurrent access of some shared resource.		\n\
								\n\
This program creates %d host threads, %d tasks and creates	\n\
a single mutex to protect access of a shared resource integer	\n\
counter located in main storage. As each host thread or MPU	\n\
task runs, it locks the mutex and increments the shared		\n\
resource counter before unlocking the mutex. Since the shared	\n\
resource is protected from concurrent accesses, the resulting	\n\
value of the counter should equal to the number of total host	\n\
threads and tasks, (%d), when the program has completed.	\n\
\n", NUM_TASKS, NUM_TASKS, NUM_TASKS * 2

#define NUM_TASKS	10

extern struct spe_program_handle mpu_task_prog;

static struct mars_context *mars_ctx;
static struct mars_task_id task_id[NUM_TASKS];
static struct mars_task_args task_args;
static uint64_t mutex_ea;
static pthread_t thread_id[NUM_TASKS];
static uint32_t shared_resource __attribute__((aligned(16)));

static void *thread(void *arg)
{
	(void)arg;
	int ret;

	ret = mars_mutex_lock(mutex_ea);
	if (ret) {
		printf("MARS mutex lock failed! (%d)\n", ret);
		pthread_exit(&ret);
	}

	shared_resource++;

	printf("HOST  : Main() - Increment Shared Resource Counter = %d\n",
		shared_resource);

	ret = mars_mutex_unlock(mutex_ea);
	if (ret) {
		printf("MARS mutex unlock failed! (%d)\n", ret);
		pthread_exit(&ret);
	}

	pthread_exit(&ret);

	return NULL;
}

int main(void)
{
	int ret, i;

	printf(INFO);

	ret = mars_context_create(&mars_ctx, 0, 0);
	if (ret) {
		printf("MARS context create failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_mutex_create(&mutex_ea);
	if (ret) {
		printf("MARS mutex create failed! (%d)\n", ret);
		return 1;
	}

	shared_resource = 0;

	printf("HOST  : Main() - Shared Resource Counter = %d\n",
		shared_resource);

	for (i = 0; i < NUM_TASKS; i++) {
		char name[MARS_TASK_NAME_LEN_MAX];

		snprintf(name, MARS_TASK_NAME_LEN_MAX, "Task %d", i);
		ret = mars_task_create(mars_ctx, &task_id[i], name,
			mpu_task_prog.elf_image, NULL);
		if (ret) {
			printf("MARS task create failed! (%d)\n", ret);
			return 1;
		}

		task_args.type.u64[0] = mutex_ea;
		task_args.type.u64[1] = mars_ptr_to_ea(&shared_resource);
		ret = mars_task_schedule(&task_id[i], &task_args, 0);
		if (ret) {
			printf("MARS task schedule failed! (%d)\n", ret);
			return 1;
		}

		pthread_create(&thread_id[i], NULL, thread, NULL);
	}

	for (i = 0; i < NUM_TASKS; i++) {
		ret = mars_task_wait(&task_id[i], NULL);
		if (ret) {
			printf("MARS task wait failed! (%d)\n", ret);
			return 1;
		}

		ret = mars_task_destroy(&task_id[i]);
		if (ret) {
			printf("MARS task destroy failed! (%d)\n", ret);
			return 1;
		}

		pthread_join(thread_id[i], NULL);
	}

	printf("HOST  : Main() - Shared Resource Counter = %d\n",
		shared_resource);

	ret = mars_mutex_destroy(mutex_ea);
	if (ret) {
		printf("MARS mutex destroy failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_context_destroy(mars_ctx);
	if (ret) {
		printf("MARS context destroy failed! (%d)\n", ret);
		return 1;
	}

	return 0;
}
